<?php

namespace Assetdocs\Incidents\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Incidents\Http\Requests\IncidentAdminWebRequest;
use Assetdocs\Incidents\Interfaces\IncidentRepositoryInterface;
use Assetdocs\Incidents\Models\Incident;

/**
 * Admin web controller class.
 */
class IncidentAdminController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'admin.web';

    /**
     * Initialize incident controller.
     *
     * @param type IncidentRepositoryInterface $incident
     *
     * @return type
     */
    public function __construct(IncidentRepositoryInterface $incident)
    {
        $this->middleware('web');
        $this->middleware('auth:admin.web');
        $this->setupTheme(config('theme.themes.admin.theme'), config('theme.themes.admin.layout'));

        $this->repository = $incident;
        parent::__construct();
    }

    /**
     * Display a list of incident.
     *
     * @return Response
     */
    public function index(IncidentAdminWebRequest $request)
    {
        $pageLimit = $request->input('pageLimit');
        if ($request->wantsJson()) {
            $incidents  = $this->repository
            ->pushCriteria(new \Assetdocs\Incidents\Repositories\Criteria\IncidentAdminCriteria())
                ->setPresenter('\\Assetdocs\\Incidents\\Repositories\\Presenter\\IncidentListPresenter')
                ->scopeQuery(function ($query) {
                    return $query->orderBy('id', 'DESC');
                })->paginate($pageLimit);

            $incidents['recordsTotal']    = $incidents['meta']['pagination']['total'];
            $incidents['recordsFiltered'] = $incidents['meta']['pagination']['total'];
            $incidents['request']         = $request->all();
            return response()->json($incidents, 200);

        }

        $this   ->theme->prependTitle(trans('incidents::incident.names').' :: ');
        return $this->theme->of('incidents::admin.incident.index')->render();
    }

    /**
     * Display incident.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return Response
     */
    public function show(IncidentAdminWebRequest $request, Incident $incident)
    {
        if (!$incident->exists) {
            return response()->view('incidents::admin.incident.new', compact('incident'));
        }

        Form::populate($incident);
        return response()->view('incidents::admin.incident.show', compact('incident'));
    }

    /**
     * Show the form for creating a new incident.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(IncidentAdminWebRequest $request)
    {

        $incident = $this->repository->newInstance([]);

        Form::populate($incident);

        return response()->view('incidents::admin.incident.create', compact('incident'));

    }

    /**
     * Create new incident.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(IncidentAdminWebRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.web');
            $incident          = $this->repository->create($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('incidents::incident.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/incidents/incident/'.$incident->getRouteKey())
            ], 201);


        } catch (Exception $e) {
            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
            ], 400);
        }
    }

    /**
     * Show incident for editing.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return Response
     */
    public function edit(IncidentAdminWebRequest $request, Incident $incident)
    {
        Form::populate($incident);
        return  response()->view('incidents::admin.incident.edit', compact('incident'));
    }

    /**
     * Update the incident.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return Response
     */
    public function update(IncidentAdminWebRequest $request, Incident $incident)
    {
        try {

            $attributes = $request->all();

            $incident->update($attributes);

            return response()->json([
                'message'  => trans('messages.success.updated', ['Module' => trans('incidents::incident.name')]),
                'code'     => 204,
                'redirect' => trans_url('/admin/incidents/incident/'.$incident->getRouteKey())
            ], 201);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/incidents/incident/'.$incident->getRouteKey()),
            ], 400);

        }
    }

    /**
     * Remove the incident.
     *
     * @param Model   $incident
     *
     * @return Response
     */
    public function destroy(IncidentAdminWebRequest $request, Incident $incident)
    {

        try {

            $t = $incident->delete();

            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('incidents::incident.name')]),
                'code'     => 202,
                'redirect' => trans_url('/admin/incidents/incident/0'),
            ], 202);

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 400,
                'redirect' => trans_url('/admin/incidents/incident/'.$incident->getRouteKey()),
            ], 400);
        }
    }
}
