<?php

namespace Assetdocs\Incidents\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Assetdocs\Incidents\Http\Requests\IncidentUserApiRequest;
use Assetdocs\Incidents\Interfaces\IncidentRepositoryInterface;
use Assetdocs\Incidents\Models\Incident;

/**
 * User API controller class.
 */
class IncidentUserApiController extends BaseController
{

    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'api';

    /**
     * Initialize incident controller.
     *
     * @param type IncidentRepositoryInterface $incident
     *
     * @return type
     */
    public function __construct(IncidentRepositoryInterface $incident)
    {
        $this->middleware('api');
        $this->middleware('jwt.auth:api');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $incident;
        parent::__construct();
    }

    /**
     * Display a list of incident.
     *
     * @return json
     */
    public function index(IncidentUserApiRequest $request)
    {
        $incidents  = $this->repository
            ->pushCriteria(new \Lavalite\Incidents\Repositories\Criteria\IncidentUserCriteria())
            ->setPresenter('\\Assetdocs\\Incidents\\Repositories\\Presenter\\IncidentListPresenter')
            ->scopeQuery(function($query){
                return $query->orderBy('id','DESC');
            })->all();
        $incidents['code'] = 2000;
        return response()->json($incidents) 
            ->setStatusCode(200, 'INDEX_SUCCESS');

    }

    /**
     * Display incident.
     *
     * @param Request $request
     * @param Model   Incident
     *
     * @return Json
     */
    public function show(IncidentUserApiRequest $request, Incident $incident)
    {

        if ($incident->exists) {
            $incident         = $incident->presenter();
            $incident['code'] = 2001;
            return response()->json($incident)
                ->setStatusCode(200, 'SHOW_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }

    }

    /**
     * Show the form for creating a new incident.
     *
     * @param Request $request
     *
     * @return json
     */
    public function create(IncidentUserApiRequest $request, Incident $incident)
    {
        $incident         = $incident->presenter();
        $incident['code'] = 2002;
        return response()->json($incident)
            ->setStatusCode(200, 'CREATE_SUCCESS');
    }

    /**
     * Create new incident.
     *
     * @param Request $request
     *
     * @return json
     */
    public function store(IncidentUserApiRequest $request)
    {
        try {
            $attributes             = $request->all();
            $attributes['user_id']  = user_id('admin.api');
            $incident          = $this->repository->create($attributes);
            $incident          = $incident->presenter();
            $incident['code']  = 2004;

            return response()->json($incident)
                ->setStatusCode(201, 'STORE_SUCCESS');
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
                'code'    => 4004,
            ])->setStatusCode(400, 'STORE_ERROR');
        }

    }

    /**
     * Show incident for editing.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return json
     */
    public function edit(IncidentUserApiRequest $request, Incident $incident)
    {
        if ($incident->exists) {
            $incident         = $incident->presenter();
            $incident['code'] = 2003;
            return response()-> json($incident)
                ->setStatusCode(200, 'EDIT_SUCCESS');;
        } else {
            return response()->json([])
                ->setStatusCode(400, 'SHOW_ERROR');
        }
    }

    /**
     * Update the incident.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return json
     */
    public function update(IncidentUserApiRequest $request, Incident $incident)
    {
        try {

            $attributes = $request->all();

            $incident->update($attributes);
            $incident         = $incident->presenter();
            $incident['code'] = 2005;

            return response()->json($incident)
                ->setStatusCode(201, 'UPDATE_SUCCESS');


        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4005,
            ])->setStatusCode(400, 'UPDATE_ERROR');

        }
    }

    /**
     * Remove the incident.
     *
     * @param Request $request
     * @param Model   $incident
     *
     * @return json
     */
    public function destroy(IncidentUserApiRequest $request, Incident $incident)
    {

        try {

            $t = $incident->delete();

            return response()->json([
                'message'  => trans('messages.success.delete', ['Module' => trans('incidents::incident.name')]),
                'code'     => 2006
            ])->setStatusCode(202, 'DESTROY_SUCCESS');

        } catch (Exception $e) {

            return response()->json([
                'message'  => $e->getMessage(),
                'code'     => 4006,
            ])->setStatusCode(400, 'DESTROY_ERROR');
        }
    }
}
