<?php

namespace Assetdocs\Incidents\Http\Controllers;

use App\Http\Controllers\Controller as BaseController;
use Form;
use Assetdocs\Incidents\Http\Requests\IncidentUserWebRequest;
use Assetdocs\Incidents\Interfaces\IncidentRepositoryInterface;
use Assetdocs\Incidents\Models\Incident;

class IncidentUserController extends BaseController
{
    /**
     * The authentication guard that should be used.
     *
     * @var string
     */
    protected $guard = 'web';

    /**
     * Initialize incident controller.
     *
     * @param type IncidentRepositoryInterface $incident
     *
     * @return type
     */
    public function __construct(IncidentRepositoryInterface $incident)
    {
        $this->middleware('web');
        $this->middleware('auth:web');
        $this->setupTheme(config('theme.themes.user.theme'), config('theme.themes.user.layout'));

        $this->repository = $incident;
        parent::__construct();
    }

    /**
     * Display a listing of the resource.
     *
     * @return Response
     */
    public function index(IncidentUserWebRequest $request)
    {
        $this->repository->pushCriteria(new \Assetdocs\Incidents\Repositories\Criteria\IncidentUserCriteria());
        $incidents = $this->repository->scopeQuery(function($query){
            return $query->orderBy('id','DESC');
        })->paginate();

        $this->theme->prependTitle(trans('incidents::incident.names').' :: ');

        return $this->theme->of('incidents::user.incident.index', compact('incidents'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     * @param Incident $incident
     *
     * @return Response
     */
    public function show(IncidentUserWebRequest $request, Incident $incident)
    {
        Form::populate($incident);

        return $this->theme->of('incidents::user.incident.show', compact('incident'))->render();
    }

    /**
     * Show the form for creating a new resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(IncidentUserWebRequest $request)
    {

        $incident = $this->repository->newInstance([]);
        Form::populate($incident);

        return $this->theme->of('incidents::user.incident.create', compact('incident'))->render();
    }

    /**
     * Display the specified resource.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(IncidentUserWebRequest $request)
    {
        try {
            $attributes = $request->all();
            $attributes['user_id'] = user_id();
            $incident = $this->repository->create($attributes);

            return redirect(trans_url('/user/incidents/incident'))
                -> with('message', trans('messages.success.created', ['Module' => trans('incidents::incident.name')]))
                -> with('code', 201);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param Request $request
     * @param Incident $incident
     *
     * @return Response
     */
    public function edit(IncidentUserWebRequest $request, Incident $incident)
    {

        Form::populate($incident);

        return $this->theme->of('incidents::user.incident.edit', compact('incident'))->render();
    }

    /**
     * Update the specified resource.
     *
     * @param Request $request
     * @param Incident $incident
     *
     * @return Response
     */
    public function update(IncidentUserWebRequest $request, Incident $incident)
    {
        try {
            $this->repository->update($request->all(), $incident->getRouteKey());

            return redirect(trans_url('/user/incidents/incident'))
                ->with('message', trans('messages.success.updated', ['Module' => trans('incidents::incident.name')]))
                ->with('code', 204);

        } catch (Exception $e) {
            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);
        }
    }

    /**
     * Remove the specified resource.
     *
     * @param int $id
     *
     * @return Response
     */
    public function destroy(IncidentUserWebRequest $request, Incident $incident)
    {
        try {
            $this->repository->delete($incident->getRouteKey());
            return response()->json([
                'message'  => trans('messages.success.deleted', ['Module' => trans('incidents::incident.name')]),
                'code'     => 202,
                'redirect' => trans_url('/user/incidents/incident/0'),
            ], 202);

        } catch (Exception $e) {

            redirect()->back()->withInput()->with('message', $e->getMessage())->with('code', 400);

        }
    }
}
